@@ -35,7 +35,7 @@ logger = logging.getLogger(__name__)
 class DataDestination(Enum):
     """Possible destinations for data routing"""
     POSTGRESQL = "postgresql"
-    MONGODB = "mongodb"
+    SQLITE = "sqlite"  # Repurposed from MONGODB
     VECTOR_DB = "vector_db"
 
 class InsightType(Enum):
@@ -44,6 +44,7 @@ class InsightType(Enum):
     HISTORICAL_STRUCTURED = "historical-structured"
     HISTORICAL_SEMANTIC = "historical-semantic"
     HYBRID = "hybrid"
+    FILE = "file"  # New for file source
 
 class DataPreprocessor:
     """Preprocesses incoming data by cleaning, normalizing, and enriching text"""
@@ -61,18 +62,12 @@ class DataPreprocessor:
         
         try:
             processed = text
-            # Step 1: Lowercase
             if self.lowercase:
                 processed = processed.lower()
-            
-            # Step 2: Remove special characters
             if self.remove_special_chars:
                 processed = self.special_char_pattern.sub('', processed)
-            
-            # Step 3: Remove extra spaces
             if self.remove_extra_spaces:
                 processed = ' '.join(processed.split())
-            
             return processed.strip()
         except Exception as e:
             logger.warning(f"Preprocessing failed for text: {text[:50]}... Error: {str(e)}")
@@ -89,15 +84,16 @@ class ContextChainConfig(BaseModel):
     llm_api_key: Optional[str] = os.getenv("LLM_API_KEY")
     prompt_config: Optional[PromptConstructionConfig] = None
     prompt_quality_threshold: float = float(os.getenv("PROMPT_QUALITY_THRESHOLD", "0.5"))
-    enable_mongo: bool = False
-    mongo_uri: str = "mongodb://localhost:27017/contextchain"
+    db_path: str = "contextchain.db"
     max_parallel_tasks: int = 5
     max_query_length: int = 1000
     max_context_length: int = 50000
     structured_indicators: List[str] = ["forecast", "prediction", "metric", "kpi", "revenue", "sales", "result", "average", "trend"]
     contextual_indicators: List[str] = ["analysis", "insight", "summary", "context", "document"]
-    default_destination: DataDestination = DataDestination.MONGODB
-    preprocessor_config: Optional[Dict[str, Any]] = None  # New: Config for DataPreprocessor
+    default_destination: DataDestination = DataDestination.SQLITE
+    preprocessor_config: Optional[Dict[str, Any]] = None
+    enable_vector: bool = True  # New: Toggle vector store
+    file_paths: Optional[List[str]] = None  # Optional default files
 
     class Config:
         arbitrary_types_allowed = True
@@ -150,24 +146,30 @@ class ContextChain:
         self.acba = AdaptiveContextBudgetingAlgorithm(max_tokens=config.max_tokens)
         prompt_cfg = config.prompt_config or PromptConstructionConfig(quality_threshold=config.prompt_quality_threshold)
         self.context_engineer = ContextEngineer(prompt_cfg)
-        self.vector_store = HybridVectorStore(config.vector_config or VectorStoreConfig())
-        self.dag_engine = DAGEngine(config.max_parallel_tasks)
-        self.storage = IntelligentStorage(mongo_uri=config.mongo_uri) if config.enable_mongo else None
-        self.preprocessor = DataPreprocessor(config.preprocessor_config or {})  # New: Initialize DataPreprocessor
+        self.vector_store = HybridVectorStore(config.vector_config or VectorStoreConfig()) if config.enable_vector else None
+        # FIXED: Properly inject shared components into DAGEngine for dependency resolution
+        # This ensures tasks like BudgetAllocationTask receive required instances (e.g., acba)
+        self.dag_engine = DAGEngine(
+            max_parallel_tasks=config.max_parallel_tasks,
+            vector_store=self.vector_store,
+            llm_optimizer=self.llm_optimizer,
+            context_engineer=self.context_engineer,
+            acba=self.acba
+        )
+        self.storage = IntelligentStorage(db_path=config.db_path)
+        self.preprocessor = DataPreprocessor(config.preprocessor_config or {})
         self.performance_stats = {
             'total_requests': 0,
             'successful_requests': 0,
             'avg_latency_ms': 0.0,
             'start_time': datetime.utcnow()
         }
-        logger.info(f"ContextChain v2.0 initialized with LLM provider: {config.llm_provider}, model: {config.llm_model}, quality_threshold: {prompt_cfg.quality_threshold}")
+        logger.info(f"ContextChain v2.0 initialized with LLM provider: {config.llm_provider}, model: {config.llm_model}, quality_threshold: {prompt_cfg.quality_threshold}, vector_enabled: {config.enable_vector}")
 
     async def initialize(self):
-        """Initialize components"""
         logger.info("Initializing ContextChain components...")
         try:
-            if self.storage:
-                await self.storage.initialize()
+            await self.storage.initialize()
             await self._register_workflows()
             await self._warmup_components()
             logger.info("ContextChain initialization completed successfully")
@@ -176,28 +178,24 @@ class ContextChain:
             raise
 
     async def _register_workflows(self):
-        """Register DAG workflows"""
         await self.dag_engine.register_workflow("simple_qa", "simple_retrieval")
 
     async def _warmup_components(self):
-        """Warm up components with health checks"""
         health_check = await self.llm_optimizer.health_check()
         if health_check['status'] != 'healthy':
             logger.warning(f"LLM backend health check failed: {health_check}")
 
     def _classify_query(self, query: str) -> str:
-        """Classify query type for routing (structured, semantic, or hybrid)"""
-        query_lower = self.preprocessor.preprocess(query).lower()  # Preprocess query
+        query_lower = self.preprocessor.preprocess(query).lower()
         if any(indicator in query_lower for indicator in self.config.structured_indicators):
-            return "structured"
+            return "relational"
         elif any(indicator in query_lower for indicator in self.config.contextual_indicators):
-            return "semantic"
+            return "vector"
         else:
             return "hybrid"
 
     @lru_cache(maxsize=100)
     async def _query_structured_db(self, query: str) -> List[Dict[str, Any]]:
-        """Fetch structured historical insights from PostgreSQL (with caching)"""
         logger.info(f"Executing structured query: {query}")
         try:
             results = []
@@ -207,9 +205,9 @@ class ContextChain:
                 results = [{"metric": "sales_trend", "value": 5.2, "unit": "percent", "period": "2024-2025"}]
             processed_results = [
                 {
-                    "content": self.preprocessor.preprocess(str(result)),  # Preprocess content
+                    "content": self.preprocessor.preprocess(str(result)),
                     "source_id": f"postgres_{uuid.uuid4()}",
-                    "metadata": {"type": "structured", "metric": result.get("metric")},
+                    "metadata": {"type": "relational", "metric": result.get("metric")},
                     "score": 1.0,
                     "embedding_quality": 1.0
                 } for result in results
@@ -219,132 +217,128 @@ class ContextChain:
             logger.warning(f"Structured query failed: {str(e)}, returning empty results")
             return []
 
-    async def generate_insights(self, query: str, context: Optional[str] = None,
+    async def _retrieve_from_file(self, file_path: str, query: str) -> List[Dict[str, Any]]:
+        import pandas as pd
+        try:
+            if file_path.endswith('.csv'):
+                df = pd.read_csv(file_path)
+            elif file_path.endswith('.json'):
+                df = pd.read_json(file_path)
+            else:
+                raise ValueError("Unsupported file type")
+            mask = df.apply(lambda row: query.lower() in ' '.join(row.astype(str)).lower(), axis=1)
+            filtered = df[mask].to_dict(orient='records')
+            return [
+                {
+                    'content': str(record),
+                    'source_id': f"file_{uuid.uuid4()}",
+                    'metadata': {'type': 'file', 'path': file_path},
+                    'score': 1.0,
+                    'embedding_quality': 1.0
+                } for record in filtered
+            ]
+        except Exception as e:
+            logger.warning(f"File retrieval failed: {str(e)}, returning empty results")
+            return []
+
+    async def generate_insights(self, query: str,
                                documents: Optional[List[Dict[str, Any]]] = None,
                                session_id: Optional[str] = None,
-                               mode: str = "auto", **kwargs) -> ContextChainResponse:
-        """Generate insights with intelligent processing"""
+                               mode: str = "auto",
+                               data_source: str = "auto",
+                               file_path: Optional[str] = None,
+                               **kwargs) -> ContextChainResponse:
         start_time = time.time()
         processing_steps = []
         session_id = session_id or str(uuid.uuid4())
         data_routing = {}
         
         try:
-            # Preprocess query
-            query = self.preprocessor.preprocess(query)
-            await self._validate_input(query, documents)
-            processing_steps.append({
-                'step': 'input_validation',
-                'status': 'completed',
-                'timestamp': datetime.utcnow()
-            })
+            preprocessed_query = self.preprocessor.preprocess(query)
+            await self._validate_input(preprocessed_query, documents)
+            processing_steps.append({'step': 'input_validation', 'status': 'completed', 'timestamp': datetime.utcnow()})
             
-            # Preprocess documents if provided
+            preprocessed_docs = None
             if documents:
-                documents = [
-                    {**doc, "content": self.preprocessor.preprocess(doc["content"])}
-                    for doc in documents
-                ]
+                preprocessed_docs = [{**doc, "content": self.preprocessor.preprocess(doc["content"])} for doc in documents]
             
-            execution_context = ExecutionContext(session_id=session_id, query=query, raw_data={'documents': documents or []})
+            execution_context = ExecutionContext(session_id=session_id, query=preprocessed_query, raw_data={'documents': preprocessed_docs or []})
             
-            complexity = self.acba.complexity_assessor.assess_complexity(query)
-            processing_steps.append({
-                'step': 'complexity_assessment',
-                'status': 'completed',
-                'complexity_score': complexity.overall_score,
-                'timestamp': datetime.utcnow()
-            })
+            complexity = self.acba.complexity_assessor.assess_complexity(preprocessed_query)
+            processing_steps.append({'step': 'complexity_assessment', 'status': 'completed', 'complexity_score': complexity.overall_score, 'timestamp': datetime.utcnow()})
             
-            if mode == "auto":
-                insight_type = self._classify_query(query)
+            if data_source != "auto":
+                insight_type = data_source
             else:
-                if mode not in ["vector", "structured", "hybrid"]:
-                    raise ValueError(f"Invalid mode: {mode}. Must be 'vector', 'structured', 'hybrid', or 'auto'.")
-                insight_type = "historical-semantic" if mode == "vector" else mode
-
-            if documents:
-                retrieved_docs = documents
-            elif mode == "structured":
-                retrieved_docs = await self._query_structured_db(query)
-                insight_type = InsightType.HISTORICAL_STRUCTURED.value
-            elif mode == "vector":
-                retrieved_docs = await self._retrieve_documents(query, execution_context)
-                insight_type = InsightType.HISTORICAL_SEMANTIC.value
+                insight_type = self._classify_query(preprocessed_query)
+            
+            if preprocessed_docs:
+                retrieved_docs = preprocessed_docs
+                insight_type_enum = InsightType.HISTORICAL_SEMANTIC.value if "vector" in insight_type else insight_type
+            elif insight_type == "relational":
+                retrieved_docs = await self._query_structured_db(preprocessed_query)
+                insight_type_enum = InsightType.HISTORICAL_STRUCTURED.value
+            elif insight_type == "vector":
+                if not self.vector_store:
+                    raise ValueError("Vector store is disabled")
+                retrieved_docs = await self._retrieve_documents(preprocessed_query, execution_context)
+                insight_type_enum = InsightType.HISTORICAL_SEMANTIC.value
+            elif insight_type == "file":
+                if not file_path:
+                    raise ValueError("file_path required for data_source='file'")
+                retrieved_docs = await self._retrieve_from_file(file_path, preprocessed_query)
+                insight_type_enum = InsightType.FILE.value
             else:
-                docs_structured = await self._query_structured_db(query)
-                docs_semantic = await self._retrieve_documents(query, execution_context)
-                retrieved_docs = docs_structured + docs_semantic
-                insight_type = InsightType.HYBRID.value if insight_type == "hybrid" else InsightType[insight_type.upper()].value
+                docs_relational = await self._query_structured_db(preprocessed_query)
+                docs_vector = await self._retrieve_documents(preprocessed_query, execution_context) if self.vector_store else []
+                retrieved_docs = docs_relational + docs_vector
+                insight_type_enum = InsightType.HYBRID.value
+            
+            if any(not doc.get('content_preprocessed') for doc in retrieved_docs if 'content_preprocessed' in doc):
+                retrieved_docs = [{**doc, 'content': self.preprocessor.preprocess(doc['content'])} for doc in retrieved_docs]
             
             execution_context.raw_data['documents'] = retrieved_docs
-            processing_steps.append({
-                'step': 'document_retrieval',
-                'status': 'completed',
-                'documents_count': len(retrieved_docs),
-                'insight_type': insight_type,
-                'timestamp': datetime.utcnow()
-            })
+            processing_steps.append({'step': 'document_retrieval', 'status': 'completed', 'documents_count': len(retrieved_docs), 'insight_type': insight_type_enum, 'timestamp': datetime.utcnow()})
             
-            budget = await self.acba.compute_optimal_budget(query, retrieved_docs, context={'session_id': session_id})
+            budget = await self.acba.compute_optimal_budget(preprocessed_query, retrieved_docs, context={'session_id': session_id})
             execution_context.budget_allocation = budget
-            processing_steps.append({
-                'step': 'budget_allocation',
-                'status': 'completed',
-                'arm_selected': budget.arm_selected,
-                'total_budget': budget.total_budget,
-                'timestamp': datetime.utcnow()
-            })
+            processing_steps.append({'step': 'budget_allocation', 'status': 'completed', 'arm_selected': budget.arm_selected, 'total_budget': budget.total_budget, 'timestamp': datetime.utcnow()})
             
             workflow_map = {
-                'structured': 'historical_analysis',
-                'semantic': 'simple_qa',
-                'hybrid': 'complex_analytical'
+                'relational': 'historical_analysis',
+                'vector': 'simple_qa',
+                'hybrid': 'complex_analytical',
+                'file': 'simple_qa'  # Reuse for file
             }
             workflow_name = workflow_map.get(insight_type, 'simple_qa')
             final_context = await self.dag_engine.execute_workflow(workflow_name, execution_context)
-            processing_steps.append({
-                'step': 'dag_execution',
-                'status': 'completed',
-                'workflow_used': workflow_name,
-                'timestamp': datetime.utcnow()
-            })
+            processing_steps.append({'step': 'dag_execution', 'status': 'completed', 'workflow_used': workflow_name, 'timestamp': datetime.utcnow()})
             
             optimized_context = await self.context_engineer.build_prompt(
-                query=query,
+                query=preprocessed_query,
                 raw_docs=retrieved_docs,
                 budget=budget,
                 complexity=complexity
             )
-            processing_steps.append({
-                'step': 'context_engineering',
-                'status': 'completed',
-                'timestamp': datetime.utcnow()
-            })
+            processing_steps.append({'step': 'context_engineering', 'status': 'completed', 'timestamp': datetime.utcnow()})
             
             llm_response = await self.llm_optimizer.generate_optimized(
                 prompt=optimized_context,
                 budget=budget,
                 stream=kwargs.get('stream', False)
             )
-            processing_steps.append({
-                'step': 'llm_generation',
-                'status': 'completed',
-                'tokens_used': llm_response.tokens_used,
-                'timestamp': datetime.utcnow()
-            })
+            processing_steps.append({'step': 'llm_generation', 'status': 'completed', 'tokens_used': llm_response.tokens_used, 'timestamp': datetime.utcnow()})
             
             total_latency_ms = (time.time() - start_time) * 1000
             quality_score = self._assess_response_quality(llm_response, complexity)
             tokens_saved = self._calculate_tokens_saved(retrieved_docs, budget)
             
-            if self.storage:
-                data_routing = await self.store_business_data(
-                    data_type="insight_response",
-                    data=llm_response.content,
-                    metadata={'session_id': session_id, 'query': query[:100], 'insight_type': insight_type},
-                    destination=self.config.default_destination.value
-                )
+            data_routing = await self.store_business_data(
+                data_type="insight_response",
+                data=llm_response.content,
+                metadata={'session_id': session_id, 'query': preprocessed_query[:100], 'insight_type': insight_type_enum},
+                destination=self.config.default_destination.value
+            )
             
             response = ContextChainResponse(
                 response=llm_response.content,
@@ -359,20 +353,18 @@ class ContextChain:
                 documents_used=len(retrieved_docs),
                 timestamp=datetime.utcnow(),
                 success=True,
-                insight_type=insight_type
+                insight_type=insight_type_enum
             )
             
-            if self.storage:
-                asyncio.create_task(self._log_interaction(query, complexity, budget, response, final_context))
+            asyncio.create_task(self._log_interaction(preprocessed_query, complexity, budget, response, final_context))
             
             self._update_performance_stats(response)
             return response
             
         except Exception as e:
-            return await self._handle_error(e, query, session_id, processing_steps, start_time)
+            return await self._handle_error(e, preprocessed_query or query, session_id, processing_steps, start_time)
 
     async def _validate_input(self, query: str, documents: Optional[List[Dict]] = None):
-        """Validate input query and documents"""
         if not query or not query.strip():
             raise ValueError("Query cannot be empty")
         if len(query) > self.config.max_query_length:
@@ -381,14 +373,11 @@ class ContextChain:
             raise ValueError(f"Context too long")
 
     async def _retrieve_documents(self, query: str, context: ExecutionContext) -> List[Dict[str, Any]]:
-        """Retrieve relevant documents from vector store"""
         try:
-            # Preprocess query before searching
-            query = self.preprocessor.preprocess(query)
             search_results = await self.vector_store.search(query=query, k=5)
             return [
                 {
-                    'content': self.preprocessor.preprocess(result.content),  # Preprocess retrieved content
+                    'content': self.preprocessor.preprocess(result.content),
                     'source_id': result.source_id,
                     'score': result.fusion_score,
                     'metadata': result.metadata,
@@ -400,20 +389,17 @@ class ContextChain:
             return []
 
     def _assess_response_quality(self, llm_response: GenerationResult, complexity: QueryComplexity) -> float:
-        """Assess response quality based on length and complexity"""
         response_length = len(llm_response.content.split())
         target_length = complexity.overall_score * 100
         length_score = min(target_length / max(response_length, 1), 1.0)
         return length_score
 
     def _calculate_tokens_saved(self, documents: List[Dict], budget: BudgetAllocation) -> int:
-        """Calculate tokens saved during optimization"""
         total_possible_tokens = sum(len(str(doc)) for doc in documents) // 4
         return max(0, total_possible_tokens - budget.generation_tokens)
 
     async def _log_interaction(self, query: str, complexity: QueryComplexity, budget: BudgetAllocation,
                               response: ContextChainResponse, context: ExecutionContext):
-        """Log interaction to storage"""
         try:
             performance_metrics = {
                 'documents_retrieved': response.documents_used,
@@ -435,7 +421,6 @@ class ContextChain:
             logger.error(f"Failed to log interaction: {str(e)}")
 
     def _update_performance_stats(self, response: ContextChainResponse):
-        """Update performance statistics"""
         self.performance_stats['total_requests'] += 1
         if response.success:
             self.performance_stats['successful_requests'] += 1
@@ -447,7 +432,6 @@ class ContextChain:
 
     async def _handle_error(self, error: Exception, query: str, session_id: str,
                            processing_steps: List[Dict], start_time: float) -> ContextChainResponse:
-        """Handle errors during processing"""
         logger.error(f"Error processing query '{query[:100]}...': {str(error)}")
         processing_steps.append({
             'step': 'error_handling',
@@ -476,9 +460,9 @@ class ContextChain:
         return error_response
 
     async def index_documents(self, documents: List[Dict[str, Any]]) -> Dict[str, Any]:
-        """Index documents in vector store"""
+        if not self.vector_store:
+            return {'status': 'error', 'error': 'Vector store is disabled'}
         try:
-            # Preprocess documents before indexing
             processed_docs = [
                 {**doc, "content": self.preprocessor.preprocess(doc["content"])}
                 for doc in documents
@@ -496,10 +480,9 @@ class ContextChain:
             return {'status': 'error', 'error': str(e)}
 
     async def get_system_status(self) -> Dict[str, Any]:
-        """Get system status and component health"""
         try:
             llm_health = await self.llm_optimizer.health_check()
-            vector_stats = self.vector_store.get_performance_stats()
+            vector_stats = self.vector_store.get_performance_stats() if self.vector_store else {"status": "disabled"}
             return {
                 'status': 'healthy',
                 'uptime_seconds': (datetime.utcnow() - self.performance_stats['start_time']).total_seconds(),
@@ -514,8 +497,6 @@ class ContextChain:
             return {'status': 'error', 'error': str(e)}
 
     def decide_data_destination(self, data_type: str, content: str, metadata: Optional[Dict] = None) -> DataDestination:
-        """Intelligent routing decision for metadata storage (fallback)"""
-        content = self.preprocessor.preprocess(content)  # Preprocess content
         content_lower = content.lower()
         if data_type in self.config.structured_indicators or any(indicator in content_lower for indicator in self.config.structured_indicators):
             return DataDestination.POSTGRESQL
@@ -526,50 +507,47 @@ class ContextChain:
 
     async def store_business_data(self, data_type: str, data: Any, metadata: Optional[Dict] = None,
                                  destination: Optional[str] = None) -> Dict[str, str]:
-        """Store business data with user-specified destination or fallback routing"""
         content = json.dumps(data) if not isinstance(data, str) else data
-        content = self.preprocessor.preprocess(content)  # Preprocess content
+        content = self.preprocessor.preprocess(content)
         routing = {}
         
         if destination:
-            if destination not in ["mongodb", "postgresql"]:
-                raise ValueError(f"Invalid destination: {destination}. Must be 'mongodb' or 'postgresql'.")
-            if destination == "mongodb" and not self.storage:
-                raise ValueError("MongoDB storage not enabled. Set enable_mongo=True in config.")
-            if destination == "mongodb":
-                doc_id = await self.storage._store_to_mongodb(data_type, content, metadata or {})
-                routing["mongodb"] = doc_id
+            if destination not in ["sqlite", "postgresql"]:
+                raise ValueError(f"Invalid destination: {destination}. Must be 'sqlite' or 'postgresql'.")
+            if destination == "sqlite":
+                doc_id = await self.storage._store_to_sqlite(data_type, content, metadata or {})
+                routing["sqlite"] = doc_id
             elif destination == "postgresql":
                 routing["postgresql"] = "handled_by_application"
         else:
             dest = self.decide_data_destination(data_type, content, metadata)
-            if dest == DataDestination.MONGODB and self.storage:
-                doc_id = await self.storage._store_to_mongodb(data_type, content, metadata or {})
-                routing["mongodb"] = doc_id
+            if dest == DataDestination.SQLITE:
+                doc_id = await self.storage._store_to_sqlite(data_type, content, metadata or {})
+                routing["sqlite"] = doc_id
             elif dest == DataDestination.POSTGRESQL:
                 routing["postgresql"] = "handled_by_application"
             elif dest == DataDestination.VECTOR_DB:
-                doc_id = await self.vector_store.index_documents([{"content": content, "metadata": metadata or {}}])
-                routing["vector_db"] = doc_id[0] if doc_id else "none"
+                if not self.vector_store:
+                    routing["vector_db"] = "disabled"
+                else:
+                    doc_id = await self.vector_store.index_documents([{"content": content, "metadata": metadata or {}}])
+                    routing["vector_db"] = doc_id[0] if doc_id else "none"
         
-        if self.storage:
-            meta_id = await self.storage._store_metadata(data_type, content, metadata or {}, destination or dest.value)
-            routing["metadata"] = meta_id
+        meta_id = await self.storage._store_metadata(data_type, content, metadata or {}, destination or dest.value)
+        routing["metadata"] = meta_id
         
         return routing
 
     async def search_context(self, query: str, k: int = 5, data_type: Optional[str] = None) -> List[Dict[str, Any]]:
-        """Search stored context in vector store"""
-        query = self.preprocessor.preprocess(query)  # Preprocess query
+        if not self.vector_store:
+            raise ValueError("Vector store is disabled")
+        query = self.preprocessor.preprocess(query)
         return await self.vector_store.search(query=query, k=k, metadata_filter={"data_type": data_type} if data_type else None)
 
     async def submit_feedback(self, session_id: str, rating: int, comments: Optional[str] = None) -> Dict[str, str]:
-        """Store feedback for a session"""
         try:
             if comments:
-                comments = self.preprocessor.preprocess(comments)  # Preprocess comments
-            if not self.storage:
-                raise ValueError("MongoDB storage not enabled for feedback")
+                comments = self.preprocessor.preprocess(comments)
             if not 1 <= rating <= 5:
                 raise ValueError("Rating must be between 1 and 5")
             feedback_id = await self.storage.store_feedback(session_id, rating, comments)
@@ -579,18 +557,16 @@ class ContextChain:
             return {"status": "error", "error": str(e)}
 
     async def close(self):
-        """Close resources"""
-        if self.storage:
-            await self.storage.close()
+        await self.storage.close()
         if hasattr(self.llm_optimizer, 'close'):
             await self.llm_optimizer.close()
         if hasattr(self.context_engineer, 'close'):
-            await self.context_engineer.close()
+            await self.context_engineer.close() 
         await self.dag_engine.close()
-        await self.vector_store.close()
+        if self.vector_store:
+            await self.vector_store.close()
 
 def create_contextchain_app(config: ContextChainConfig = None, llm_client: Optional[BaseLLMClient] = None) -> FastAPI:
-    """Library entry point to create a FastAPI application with ContextChain integration"""
     config = config or ContextChainConfig()
     context_chain = ContextChain(config, llm_client=llm_client)
     
@@ -637,30 +613,33 @@ def create_contextchain_app(config: ContextChainConfig = None, llm_client: Optio
     @app.get("/metrics")
     async def get_metrics():
         try:
-            return {
-                "status": "success",
-                "metrics": context_chain.performance_stats,
-                "vector_store_stats": context_chain.vector_store.get_performance_stats(),
-                "llm_stats": await context_chain.llm_optimizer.get_performance_stats()
+            stats = {
+                "performance": context_chain.performance_stats,
+                "vector_store": context_chain.vector_store.get_performance_stats() if context_chain.vector_store else {"status": "disabled"},
+                "llm": await context_chain.llm_optimizer.get_performance_stats()
             }
+            return {"status": "success", "metrics": stats}
         except Exception as e:
             logger.error(f"Metrics retrieval failed: {str(e)}")
             raise HTTPException(status_code=500, detail=str(e))
     
     @app.post("/query")
     @app.state.limiter.limit("10/minute")
-    async def query_insights(request: Request, query: str, context: Optional[str] = None,
+    async def query_insights(request: Request, query: str,
                              documents: Optional[List[Dict[str, Any]]] = None,
                              session_id: Optional[str] = None,
                              mode: str = "auto",
+                             data_source: str = "auto",  # New
+                             file_path: Optional[str] = None,  # New
                              stream: bool = False):
         try:
             response = await context_chain.generate_insights(
                 query=query,
-                context=context,
                 documents=documents,
                 session_id=session_id,
                 mode=mode,
+                data_source=data_source,
+                file_path=file_path,
                 stream=stream
             )
             if stream:
